#include <bits/stdc++.h>

/* ============================================================= */

template <class ...Args>
bool read(Args& ...args)
{ return (bool)(std::cin >> ... >> args); }

template <class ...Args>
void write(Args&& ...args)
{ (std::cout << ... << std::forward<Args>(args)); }

template <class ...Args>
void writeln(Args&& ...args)
{ (std::cout << ... << std::forward<Args>(args)) << '\n'; }

/* ============================================================= */

using graph_t = std::vector<std::vector<int>>;

bool path_dfs(const graph_t& g, int v, int p, int b, std::vector<int>& path) {
    path.push_back(v);
    if (v == b)
        return true;
    for (int u : g[v])
        if (u != p && path_dfs(g, u, v, b, path))
            return true;
    path.pop_back();
    return false;
}

template <class Func>
bool check_dfs(const graph_t& g, int v, int p, int cur, Func& check) {
    if (!check(v, cur))
        return false;
    for (int u : g[v])
        if (u != p && !check_dfs(g, u, v, cur+1, check))
            return false;
    return true;
}

template <class Func>
bool check_dist(const graph_t& g, int v, Func&& check) {
    check_dfs(g, v, -1, 0, check);
}

std::pair<int, int> middle(const graph_t& g, int a, int b) {
    std::vector<int> path;
    path.reserve(g.size());
    path_dfs(g, a, -1, b, path);
    if (path.size() % 2 == 0)
        return {-1, -1};
    int n = path.size();
    return { path[n/2], n/2 };
}

int main() {
    std::ios_base::sync_with_stdio(false);
    std::cin.tie(nullptr);
    std::cout.tie(nullptr);
    std::fixed(std::cout).precision(10);

    int n, m; read(n, m);
    graph_t g(n);
    for (int i = 0; i < n-1; ++i) {
        int a, b; read(a, b); --a, --b;
        g[a].push_back(b);
        g[b].push_back(a);
    }

    std::vector<int> cs(m);
    for (int& x : cs)
        read(x), --x;

    if (m == 1) {
        writeln("YES");
        writeln(cs[0]+1);
    } else {
        auto [mid, dst] = middle(g, cs[0], cs[1]);
        if (mid == -1) {
            writeln("NO");
            return 0;
        }

        std::sort(cs.begin(), cs.end());

        bool good = check_dist(g, mid, [&, dd=dst](int v, int d) {
            return !std::binary_search(cs.begin(), cs.end(), v) || d == dd;
        });

        if (good) {
            writeln("YES");
            writeln(mid+1);
        } else {
            writeln("NO");
        }
    }

    return 0;
}