#include <iostream>
#include <vector>
#include <cassert>
#include <algorithm>

using namespace std;
#define BIG 1000000

const int maxN = 1010;
char f[maxN][maxN];

int n, m;

bool is_castle(char c) {
    return c > 'A' && c <= 'Z';
}

void fill_castle(int i1, int j1, int i2, int j2, char c) {
    for(int i = i1; i < i2; i++) {
        for(int j = j1; j < j2; j++) {
            if(f[i][j] != c) {
                f[i][j] = c - 'A' + 'a';
            }
        }
    }
}

pair<int, int> find_seg(int i, int j) {
    pair<int, int> seg = make_pair(j, j);
    for(int dj = j; dj < m; dj++) {
        if(!is_castle(f[i][dj])) {
            seg.second = dj;
        } else {
            break;
        }
    }
    for(int dj = j; dj >= 0; dj--) {
        if(!is_castle(f[i][dj])) {
            seg.first = dj;
        } else {
            break;
        }
    }
    return seg;
}

pair<pair<int, int>, pair<int, int>> fillA(int ai, int aj) {
    vector<pair<int, int> > seg(n);
    int mx = 1;
    int j1 = aj;
    int j2 = aj;
    int i1 = ai;
    int i2 = ai;
    for(int i = 0; i < n; i++) {
        seg[i] = find_seg(i, aj);
    }
    for(int i = ai + 1; i < n; i++) {
        seg[i] = make_pair(max(seg[i].first, seg[i - 1].first), min(seg[i].second, seg[i - 1].second));
    }
    for(int i = ai - 1; i >= 0; i--) {
        seg[i] = make_pair(max(seg[i].first, seg[i + 1].first), min(seg[i].second, seg[i + 1].second));
    }
    for(int u = ai; u < n; u++) {
        if(is_castle(f[aj][u])) {
            break;
        }
        for(int d = ai; d >= 0; d--) {
            if(is_castle(f[aj][d]))
                break;
            pair<int, int> cur_seg = make_pair(max(seg[u].first, seg[d].first), min(seg[u].second, seg[d].second));
            int area = (u - d + 1) * (cur_seg.second - cur_seg.first + 1);
            if(area > mx) {
                mx = area;
                j1 = cur_seg.first;
                j2 = cur_seg.second;
                i1 = d;
                i2 = u;
            }
        }
    }
    fill_castle(i1, j1, i2 + 1, j2 + 1, 'A');
    return make_pair(make_pair(i1, i2), make_pair(j1, j2));
}


int count_c(int i1, int j1, int i2, int j2) {
    int res = 0;
    for(int i = i1; i < i2; i++) {
        for(int j = j1; j < j2; j++) {
            if(is_castle(f[i][j])) {
                res++;
            }
        }
    }
    return res;
}


void filll(int i1, int j1, int i2, int j2, int all_c) {
    if(i1 == i2 || j1 == j2)
        return;
    int sti = -1;
    int str_cnt = 0;
    for(int i = i1; i < i2; i++) {
        int c_cnt = 0;
        for(int j = j1; j < j2; j++) {
            if(is_castle(f[i][j])) {
                c_cnt ++;
            }
        }
        if(c_cnt > 0) {
            sti = i;
            str_cnt = c_cnt;
            break;
        }
    }
    assert(sti != -1);

    int filled_cnt = 0;
    int prev_j = j1;
    for(int j = j1; j < j2; j++) {
        if(is_castle(f[sti][j])) {
            if(filled_cnt != str_cnt - 1) {
                if(str_cnt == all_c) {
                    fill_castle(i1, prev_j, i2, j + 1, f[sti][j]);
                } else {
                    fill_castle(i1, prev_j, sti + 1, j + 1, f[sti][j]);
                }
                filled_cnt++;
                prev_j = j + 1;
            } else {
                if(str_cnt == all_c) {
                    fill_castle(i1, prev_j, i2, j2, f[sti][j]);
                } else {
                    fill_castle(i1, prev_j, sti + 1, j2, f[sti][j]);
                }
                break;
            }
        }
    }
    if(sti != i2 - 1 && all_c != str_cnt) {
        filll(sti + 1, j1, i2, j2, all_c - str_cnt);
    }
}

void print() {
    for(int i = 0 ; i < n; i++) {
        printf("%s\n", f[i]);
    }
}

int main() {
    //freopen("input", "r", stdin);
    scanf("%d%d", &n, &m);
    for(int i = 0; i < n; i++) {
        scanf("%s", f[i]);
    }

    int ai = 0;
    int aj = 0;
    for(int i = 0; i < n; i++) {
        for (int j = 0; j < m; j++)
            if (f[i][j] == 'A') {
                ai = i;
                aj = j;
            }
    }


    auto res = fillA(ai, aj);

    int i1 = res.first.first;
    int i2 = res.first.second;
    int j1 = res.second.first;
    int j2 = res.second.second;
    filll(0, 0, i1, m, count_c(0, 0, i1, m));

    filll(i1, 0, n, j1, count_c(i1, 0, n, j1));

    filll(i1, j2 + 1, i2 + 1, m, count_c(i1, j2 + 1, i2 + 1, m));

    filll(i2 + 1, j1, n, m, count_c(i2 + 1, j1, n, m));

    for(int i = 0; i < n; i++) {
        printf("%s\n", f[i]);
    }
    return 0;
}