#include <bits/stdc++.h>

using namespace std;

using ll = long long;

static int dx[] = {1, -1, 0, 0}, dy[] = {0, 0, 1, -1};

const int inf = int(1e9);

struct edge
{
	int to, cap, old_cap;
};

struct flow
{
	int n, s, t;
	vector<vector<int>> g;
	vector<edge> e;

	flow(int n_)
	{
		n = n_;
		s = n - 2;
		t = n - 1;
		g.resize(n);
	}

	void add_edge(int from, int to, int cap)
	{
		g[from].push_back(e.size());
		e.push_back(edge{to, cap, cap});
		g[to].push_back(e.size());
		e.push_back(edge{from, 0, 0});
	}

	vector<char> used;

	int dfs(int v, int push)
	{
		if (used[v])
			return 0;
		if (v == t || push == 0)
			return push;
		used[v] = 1;

		for (int num : g[v])
		{
			const int to = e[num].to, cap = e[num].cap;
			if (cap > 0)
			{
				int pushed = dfs(to, min(cap, push));
				if (pushed > 0)
				{
					e[num].cap -= pushed;
					e[num ^ 1].cap += pushed;
					return pushed;
				}
			}
		}

		return 0;
	}

	int add_flow()
	{
		used.assign(n, 0);
		return dfs(s, inf);
	}

	int max_flow()
	{
		int ans = 0;
		while (int add = add_flow())
			ans += add;
		return ans;
	}

	vector<pair<int, int>> used_edges()
	{
		vector<pair<int, int>> ans;
		for (int i = 0; i < n; i++)
		{
			for (int num : g[i])
			{
				if (e[num].old_cap == 0)
					continue;

				for (int x = e[num].cap; x < e[num].old_cap; x++)
					ans.emplace_back(i, e[num].to);
			}
		}

		return ans;
	}
};

void dfs(int x, vector<multiset<pair<int, int>>> &g, vector<int> &path, int last)
{
	while (!g[x].empty())
	{
		auto it = g[x].begin();
		if (last == 0)
		{
			it = g[x].end();
			--it;
		}

//		for (auto pr : g[x])
//			cerr << pr.first << ' ' << pr.second << endl;

//		cerr << it -> first << ' ' << last << endl;
		assert(it -> first != last);
		const int to = (it -> second);

		g[x].erase(it);
		g[to].erase(g[to].find(pair<int, int>(last ^ 1, x)));
		dfs(to, g, path, last ^ 1);
	}
	path.push_back(x);
}

void solve(istream &cin = std::cin, ostream &cout = std::cout)
{
	int w, h;

	cin >> w >> h;
	w++;

	vector<int> cnt(w * (h + 1));

	vector<string> field(h++);

	for (auto &row : field)
		cin >> row;

	int cntx = 0;

	for (int y = 0; y < field.size(); y++)
		for (int x = 0; x < field[y].size(); x++)
			if (field[y][x] == 'X')
			{
				cntx++;

				for (int i = y; i <= y + 1; i++)
					for (int j = x; j <= x + 1; j++)
						cnt[i * w + j]++;
			}

	vector<tuple<int, int, int>> edges;

	for (int y = 0; y < h; y++)
		for (int x = 0; x < w; x++)
			if ((x + y) % 2 == 0)
			{
				if (cnt[y * w + x] > 0)
					for (int i = 0; i < 4; i++)
						if (0 <= y + dy[i] && y + dy[i] < h &&
						    0 <= x + dx[i] && x + dx[i] < w)
							edges.emplace_back(y * w + x, (y + dy[i]) * w + x + dx[i], inf);
				if (cnt[y * w + x] > 0)
					edges.emplace_back(cnt.size(), y * w + x, cnt[y * w + x]);
			}
			else if (cnt[y * w + x] > 0)
				edges.emplace_back(y * w + x, cnt.size() + 2, cnt[y * w + x]);
	edges.emplace_back(cnt.size() + 1, cnt.size(), cntx * 2 - 1);

	cerr << "built" << endl;

	int n = cnt.size() + 3;

	flow flow(n);
	for (auto e : edges)
		flow.add_edge(get<0>(e), get<1>(e), get<2>(e));

	cerr << flow.max_flow() << endl;

	auto ue = flow.used_edges();

	vector<multiset<pair<int, int>>> gr(cnt.size());

	for (int y = 0; y < field.size(); y++)
		for (int x = 0; x < field[y].size(); x++)
			if (field[y][x] == 'X')
			{
				gr[y * w + x].emplace(0, (y + 1) * w + x + 1);
				gr[(y + 1) * w + x + 1].emplace(0, y * w + x);

				gr[y * w + x + 1].emplace(0, (y + 1) * w + x);
				gr[(y + 1) * w + x].emplace(0, y * w + x + 1);
			}

	auto unsh = [w, h](int q) {
		return pair<int, int>(q % w, q / w);
	};

	for (auto it : ue)
		if (it.first < cnt.size() && it.second < cnt.size())
		{
			gr[it.first].emplace(1, it.second);
			gr[it.second].emplace(1, it.first);
		}

	vector<int> path;

	int st = 0;

	while (gr[st].size() % 2 == 0)
		st++;

	dfs(st, gr, path, 1);
	reverse(path.begin(), path.end());

	cout << path.size() - 1 << endl;
	for (auto it : path)
	{
		auto q = unsh(it);
		cout << q.first << ' ' << q.second << endl;
	}
}

int main()
{
	ios_base::sync_with_stdio(false);
	cin.tie(nullptr);

#ifdef LOCAL
	time_t st = clock();

	ifstream fin("../input.txt");

	solve(fin);

	cout << "clock: " << (clock() - st) / (double) CLOCKS_PER_SEC << endl;
#else
	solve();
#endif
	return 0;
}
