#include <bits/stdc++.h>

using namespace std;

using ll = long long;

const int let = 26;

char lower(char ch)
{
	if ('a' <= ch && ch <= 'z')
		return ch;
	if ('A' <= ch && ch <= 'Z')
		return ch + ('a' - 'A');
	assert(false);
	return -1;
}

bool upper(char ch)
{
	return 'A' <= ch && ch <= 'Z';
}

void expand_left(vector<string> &grid, int col, int l, int r)
{
	if (l >= r)
		return;

	assert(col > 0);
	for (int i = l; i < r; i++)
		grid[i][col] = lower(grid[i][col - 1]);
}

void expand_right(vector<string> &grid, int col, int l, int r)
{
	if (l >= r)
		return;

	assert(col + 1 < int(grid[0].size()));
	for (int i = l; i < r; i++)
		grid[i][col] = lower(grid[i][col + 1]);
}

bool try_fill(vector<string> &grid, int col, int l, int r)
{
	char who = -1;
	for (int i = l; i < r; i++)
	{
		if (upper(grid[i][col]))
		{
			who = grid[i][col];
			break;
		}
	}

	if (who == -1)
		return false;

	for (int i = l; i < r; i++)
	{
		if (grid[i][col] == '.')
			grid[i][col] = who + ('a' - 'A');
		else
			who = grid[i][col];
	}

	return true;
}

bool fill_table(vector<string> &grid, int l, int r, int d, int u)
{
	if (l >= r || d >= u)
		return true;

	bool ok = false;
	for (int i = l; i < r; i++)
		for (int j = d; j < u; j++)
			ok |= upper(grid[i][j]);

	if (!ok)
		return false;

	int start = d;
	while (start < u && !try_fill(grid, start, l, r))
		start++;

	assert(start < u);

	for (int i = start + 1; i < u; i++)
	{
		if (!try_fill(grid, i, l, r))
			expand_left(grid, i, l, r);
	}

	for (int i = start - 1; i >= d; i--)
		expand_right(grid, i, l, r);

	return true;
}

void solve(istream &cin = std::cin, ostream &cout = std::cout)
{
	int n, m;
	cin >> n >> m;

	vector<string> grid(n);
	for (auto &s : grid)
		cin >> s;

	vector<int> x(let, -1), y(let, -1);
	for (int i = 0; i < n; i++)
	{
		for (int j = 0; j < m; j++)
		{
			const char ch = grid[i][j];
			if ('A' <= ch && ch <= 'Z')
			{
				x[ch - 'A'] = i;
				y[ch - 'A'] = j;
			}
		}
	}

	int best = -1, ml = -1, mr = -1, md = -1, mu = -1;

	for (int up = 0; up <= let; up++)
		for (int down = 0; down <= let; down++)
			for (int left = 0; left <= let; left++)
				for (int right = 0; right <= let; right++)
				{
					bool ok = true;
					for (int v : {up, down, left, right})
						ok &= (v == let || x[v] != -1);

					if (!ok)
						continue;

					int l = (left == let ? 0 : x[left] + 1);
					int r = (right == let ? n : x[right]);
					int d = (down == let ? 0 : y[down] + 1);
					int u = (up == let ? m : y[up]);

					if (l >= r || d >= u)
						continue;

					for (int i = 1; i < let && ok; i++)
					{
						if (x[i] == -1)
							continue;

						ok &= (x[i] < l || x[i] >= r || y[i] < d || y[i] >= u);
					}

					if (!ok)
						continue;

					int cur = (r - l) * (u - d);
					if (best < cur)
					{
						best = cur;
						ml = l;
						mr = r;
						mu = u;
						md = d;
					}
				}

	cerr << ml << ' ' << mr << ' ' << md << ' ' << mu << endl;

	for (int x = ml; x < mr; x++)
		for (int y = md; y < mu; y++)
		{
			if (grid[x][y] == '.')
				grid[x][y] = 'a';
			else
				assert(grid[x][y] == 'A');
		}

	fill_table(grid, 0, ml, md, mu);
	fill_table(grid, ml, mr, 0, md);
	fill_table(grid, ml, mr, mu, m);
	fill_table(grid, mr, n, md, mu);

	if (!fill_table(grid, 0, ml, 0, md))
	{
		for (int i = md - 1; i >= 0; i--)
			expand_right(grid, i, 0, ml);
	}
	if (!fill_table(grid, mr, n, 0, md))
	{
		for (int i = md - 1; i >= 0; i--)
			expand_right(grid, i, mr, n);
	}
	if (!fill_table(grid, 0, ml, mu, m))
	{
		for (int i = mu; i < m; i++)
			expand_left(grid, i, 0, ml);
	}
	if (!fill_table(grid, mr, n, mu, m))
	{
		for (int i = mu; i < m; i++)
			expand_left(grid, i, mr, n);
	}

	for (auto s : grid)
		cout << s << endl;
}

int main()
{
	ios_base::sync_with_stdio(false);
	cin.tie(nullptr);

#ifdef LOCAL
	time_t st = clock();

	ifstream fin("../input.txt");

	solve(fin);

	cout << "clock: " << (clock() - st) / (double) CLOCKS_PER_SEC << endl;
#else
	solve();
#endif
	return 0;
}
